@@ -31,7 +31,7 @@
     https://hackage.haskell.org/package/genvalidity ):
 
     > instance Validity Prime where
-    >     validate (Prime n) = isPrime n <?@> "The 'Int' is prime."
+    >     validate (Prime n) = check (isPrime n) "The 'Int' is prime."
 
     If certain typeclass invariants exist, you can make these explicit in the
     validity instance as well.
@@ -48,8 +48,11 @@ module Data.Validity
     , annotate
     , delve
     , decorate
+    , decorateList
     , invalid
     , valid
+    , validateNotNaN
+    , validateNotInfinite
     -- * Utilities
     -- ** Utilities for validity checking
     , isValid
@@ -60,6 +63,8 @@ module Data.Validity
     , Validation(..)
     , ValidationChain(..)
     , checkValidity
+    , validationIsValid
+    , prettyValidate
     , prettyValidation
     -- * Re-exports
     , Monoid(..)
@@ -249,6 +254,13 @@ delve = flip annotate
 decorate :: String -> Validation -> Validation
 decorate = flip annotateValidation
 
+-- | Decorate a piecewise validation of a list with their location in the list
+decorateList :: [a] -> (a -> Validation) -> Validation
+decorateList as func = mconcat $
+  flip map (zip [0..] as) $ \(i, a) ->
+  decorate (unwords ["The element at index", show (i :: Integer), "in the list"]) $
+  func a
+
 -- | Construct a trivially invalid 'Validation'
 --
 -- Example:
@@ -339,17 +351,8 @@ instance ( Validity a
 -- If the empty list should not be considered valid as part of your custom data
 -- type, make sure to write a custom @Validity instance@
 instance Validity a => Validity [a] where
-    validate =
-        mconcat .
-        map
-            (\(ix, e) ->
-                 annotate e $
-                 unwords
-                     [ "The element at index"
-                     , show (ix :: Integer)
-                     , "in the list"
-                     ]) .
-        zip [0 ..]
+    validate = flip decorateList validate
+
 #if MIN_VERSION_base(4,9,0)
 -- | A nonempty list is valid if all the elements are valid.
 --
@@ -425,29 +428,19 @@ instance Validity Word32 where
 instance Validity Word64 where
     validate = trivialValidation
 
--- | NOT trivially valid:
---
--- * NaN is not valid.
--- * Infinite values are not valid.
+-- | Trivially valid:
 instance Validity Float where
-    validate f =
-        mconcat
-            [ declare "The Float is not Nan." $ not (isNaN f)
-            , declare "The Float is not infinite." $ not (isInfinite f)
-            , declare "The Float is not zegative zero." $ not (isNegativeZero f)
-            ]
+    validate = trivialValidation
 
--- | NOT trivially valid:
---
--- * NaN is not valid.
--- * Infinite values are not valid.
+-- | Trivially valid:
 instance Validity Double where
-    validate d =
-        mconcat
-            [ declare "The Double is not NaN." $ not (isNaN d)
-            , declare "The Double is not infinite." $ not (isInfinite d)
-            , declare "The Double is not zegative zero." $ not (isNegativeZero d)
-            ]
+    validate = trivialValidation
+
+validateNotNaN :: RealFloat a => a -> Validation
+validateNotNaN d = declare "The Double is not NaN." $ not (isNaN d)
+
+validateNotInfinite :: RealFloat a => a -> Validation
+validateNotInfinite d = declare "The Double is not infinite." $ not (isInfinite d)
 
 -- | Trivially valid
 --
@@ -468,12 +461,19 @@ instance Validity Natural where
 #endif
 -- | Valid if the contained numbers are valid and the denominator is
 -- strictly positive.
-instance (Num a, Ord a, Validity a) => Validity (Ratio a) where
+instance (Validity a, Ord a, Num a, Integral a) => Validity (Ratio a) where
     validate (n :% d) =
         mconcat
             [ annotate n "The numerator"
             , annotate d "The denominator"
             , declare "The denominator is strictly positive." $ d > 0
+            , declare "The ratio is normalised" $
+                case d of
+                  0 -> False
+                  _ ->
+                    let g = gcd n d
+                        n' :% d' = (n `quot` g) :% (d `quot` g)
+                    in n' :% d' == n :% d
             ]
 
 -- | Valid according to the contained 'Integer'.
@@ -550,12 +550,30 @@ checkValidity a =
         Validation [] -> Right a
         Validation errs -> Left errs
 
--- | validate a given value, and return a nice error if the value is invalid.
-prettyValidation :: Validity a => a -> Either String a
-prettyValidation a =
-    case checkValidity a of
-        Right a_ -> Right a_
-        Left errs -> Left $ intercalate "\n" $ map (errCascade . toStrings) errs
+-- | Check if a 'Validation' concerns a valid value.
+validationIsValid :: Validation -> Bool
+validationIsValid v = case v of
+    Validation [] -> True
+    _ -> False
+
+-- | Validate a given value
+--
+-- This function will return a nice error if the value is invalid.
+-- It will return the original value in 'Right' if it was valid,
+-- as evidence that it has been validated.
+prettyValidate :: Validity a => a -> Either String a
+prettyValidate a = case prettyValidation $ validate a of
+    Just e -> Left e
+    Nothing -> Right a
+
+-- | Render a `Validation` in a somewhat pretty way.
+--
+-- This function will return 'Nothing' if the 'Validation' concerned a valid value.
+prettyValidation :: Validation -> Maybe String
+prettyValidation v =
+    case v of
+        Validation [] -> Nothing
+        Validation errs -> Just $ intercalate "\n" $ map (errCascade . toStrings) errs
   where
     toStrings (Violated s) = ["Violated: " ++ s]
     toStrings (Location s vc) = s : toStrings vc
